;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xf6aa77357ca21d0f6a08571d9a7eb768 . "./code.cg2js/cg-acache.fasl")
(#xbe5681cb353e6be80568e54f51a5fd33 . "./code.cg2js/cg-base.fasl")
(#xf2f37713e6326fef0c4ad699555a50ea . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x0bd6b1bed36809ef8381246036ad1c82 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x62fa36eda453f1291539d8b16c580e74 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x9df550d66a8a50fc6bb876adb42a73c2 . "./code.cg2js/cg-button.fasl")
(#xeb437c1e713cf222b8a3468d286b351f . "./code.cg2js/cg-calendar.fasl")
(#xe25d3a90a09249c08cb5e6d3eccc744e . "./code.cg2js/cg-caret.fasl")
(#xa97d28831ba922860f0921116f0a707d . "./code.cg2js/cg-chart-or-plot.fasl")
(#x009d6bca503d05915660feab0dee58a8 . "./code.cg2js/cg-chart-widget.fasl")
(#x406e97b972e09ef34d0b3387dfba4bd4 . "./code.cg2js/cg-check-box.fasl")
(#x1e16f93ad10dbc93c1aacf838f712c3e . "./code.cg2js/cg-choice-list.fasl")
(#xd0d070fc4f1c9584fc3dd38884086714 . "./code.cg2js/cg-choose-printer.fasl")
(#x96cc74d92a6b1e1f6b96a6ed4a304ffd . "./code.cg2js/cg-class-grid.fasl")
(#xeeaf95386343c81d4e0dfc19c956d10a . "./code.cg2js/cg-class-slot-grid.fasl")
(#x6d0a3fac1e668f2c9a1b69c97d6fcad3 . "./code.cg2js/cg-class-support.fasl")
(#x679c7bc6ebbed7f027f639eaf816471b . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xac59188a54afc8dd2e0fc44740887348 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x986929ee7764fadd222aeb856d5a9bec . "./code.cg2js/cg-clipboard.fasl")
(#xbafe2eba987f049977958291b140c16d . "./code.cg2js/cg-color-dialog.fasl")
(#xa0e3263ae2c788184f9ad6eb0d9a722d . "./code.cg2js/cg-combo-box.fasl")
(#x6385f33721a74c29508b5be13ae133e3 . "./code.cg2js/cg-common-control.fasl")
(#xbf811d24ff6345d2c25b749dec202814 . "./code.cg2js/cg-comtab.fasl")
(#x06ecd3cdec32cdd2f8d6a684dc5c57a3 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xa7e627acbb54078d78662cac06aebe3f . "./code.cg2js/cg-curve.fasl")
(#x8412fe924002f220e18f1d5a7c2be77d . "./code.cg2js/cg-dde-utils.fasl")
(#x10f63f91a2a9e0d0b7afa8ec369ed5da . "./code.cg2js/cg-dev.fasl")
(#x5e9c84da1adc78ca38a17e55386d2f30 . "./code.cg2js/cg-dialog-item.fasl")
(#x3912795a5999b1f8324ce9349468dc13 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xfef0bd3f63f3647b3682a6006ac4887c . "./code.cg2js/cg-directory-dialog.fasl")
(#x89471537c0c06fb352ad41c452c2e4e8 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x07a63066610e0c5f8a77735f8866482f . "./code.cg2js/cg-drag-and-drop.fasl")
(#x11e985ecb92e683d75cc51b077f67cf5 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x3d937bd66653274a35f948041265feba . "./code.cg2js/cg-drawable.fasl")
(#xeb0d97a1988cd8a996e52b4f7fc978b7 . "./code.cg2js/cg-dropping-outline.fasl")
(#xe6ad4efa2304ea93b950ea3a38d65ce2 . "./code.cg2js/cg-edit-in-place.fasl")
(#x7b388a64d0a63c4e2bdd1ce08889ad96 . "./code.cg2js/cg-editable-text.fasl")
(#xb7b774b2fdad2a467502c7e6a2ad42c2 . "./code.cg2js/cg-file-dialog.fasl")
(#xa9d5beca87d2e218a9281df5b702b4c4 . "./code.cg2js/cg-fill-texture.fasl")
(#xd2dff7b5111f68a2a191f8db678a15ee . "./code.cg2js/cg-find-string-dialog.fasl")
(#xd5923dc47a77c410eba105ad18625553 . "./code.cg2js/cg-font-dialog.fasl")
(#x406acf3531f87067bea41209116000dd . "./code.cg2js/cg-gesture-emulation.fasl")
(#xb19396daed78173505267438c04ff5a6 . "./code.cg2js/cg-get-pixmap.fasl")
(#x57ce00a8f2393438b4ab63617040000e . "./code.cg2js/cg-get-position.fasl")
(#x4901d08f8778208ed117ffb965211331 . "./code.cg2js/cg-graphics-context.fasl")
(#x8b19b0502b3c6c079c383a0073eb8fc9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x480c85878417ec0f2ecddec5bdaf883c . "./code.cg2js/cg-grid-widget.fasl")
(#xd0a029ccb729b3a569c8b1c9634107df . "./code.cg2js/cg-group-box.fasl")
(#x4c6efa6c24bd9fc8802e362cf34572f8 . "./code.cg2js/cg-header-control.fasl")
(#x686b5d8360bad9f90144fea1e74d9d4a . "./code.cg2js/cg-hotspot.fasl")
(#x3efd109124f5e3a0b70d1619f94b0bc5 . "./code.cg2js/cg-html-dialog.fasl")
(#xfad72a257bc2c9150a4c835fe0f25b6f . "./code.cg2js/cg-html-widget.fasl")
(#x7b735048789cd965df35a41e7a662df1 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xd069448651f9a566e75433ce5aee4a96 . "./code.cg2js/cg-icon.fasl")
(#xcd6f857311814bff3c3ff5f944a830a3 . "./code.cg2js/cg-ie.fasl")
(#x23fe2be41b34b69c26e366611df6a546 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x657e485e6957b51e8ebc85779fbeb699 . "./code.cg2js/cg-intersect.fasl")
(#x7dc66e1fa6c83122640cdf62cbd2bd26 . "./code.cg2js/cg-item-list.fasl")
(#xe3e7c25faf3100d99f646ddbf8c8a2cf . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xdb0f0b105d5e7072395e033632f81402 . "./code.cg2js/cg-lamp.fasl")
(#x0c8ea6144d8bd037392a7ee9a764831c . "./code.cg2js/cg-layout.fasl")
(#xe3dc01529043e4530c9d53c63b1c15aa . "./code.cg2js/cg-lettered-menu.fasl")
(#x5f05e77d604324ab56cbfef5a69bb7e4 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x8a6635ee177afa78904f543dc3f03fee . "./code.cg2js/cg-lisp-text.fasl")
(#x4dec8f6dee5c6c19918acf1b305fb52b . "./code.cg2js/cg-lisp-widget.fasl")
(#xd9a33a768d7b1cf9671491f002129014 . "./code.cg2js/cg-list-view.fasl")
(#x5ba8b690c8410cf17ebeffd51598191e . "./code.cg2js/cg-mci.fasl")
(#x488ea440c3650fc0f26e5dd07725a1d2 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x9350470b2d14063556407e4c4c56a4ff . "./code.cg2js/cg-menu.fasl")
(#xf5dcd09c78796ce30e6aa98c9aee5b9b . "./code.cg2js/cg-message-dialog.fasl")
(#x9b82bc55afc9c709226f77c36ca36a8a . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x6f40d51ae37490dc8365266bd5d03c64 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x7bf17717c8e717089f74b098f3847832 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xecbd038fa1da9f0eac883928fa88b8d1 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x1ca24abdf1840091171dba0ee425ed88 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x7c04cd5eb4714f94f461ac9a81a54a2c . "./code.cg2js/cg-nodes.fasl")
(#x1cbf58a23ce83b024cf282e316145171 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xc9306d41c77828af847d1d6cffdafa3b . "./code.cg2js/cg-object-editor.fasl")
(#x7d075901eb273293d94f9b50ae8fde33 . "./code.cg2js/cg-ocx.fasl")
(#x2bb07745364ec86db51de786747cc432 . "./code.cg2js/cg-os-widget.fasl")
(#xe40aa1bce9089a93f3eeb626ba0d4660 . "./code.cg2js/cg-os-window.fasl")
(#x54ffb7f620060f1eec2ee4d170eb2eef . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x1db546fef159d3cba0f1a7f3e31ed9d7 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xb45119b436e8bb3176d7e083b15c72c7 . "./code.cg2js/cg-outline.fasl")
(#xfa383fb59eb82e727332d2f476947d85 . "./code.cg2js/cg-palette.fasl")
(#x12d21d6faacb1ae50dad2fdcd68aa1a7 . "./code.cg2js/cg-paren-matching.fasl")
(#x8fd2a472349481a555c472e22d1752b0 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc3e21f37dd0a30a74bc336e93cd1db38 . "./code.cg2js/cg-picture-widget.fasl")
(#x18eb91f9abe2d66146425f970db9ccde . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xa7a7a2d1b008f90505235eaf45dfc03f . "./code.cg2js/cg-pixmap-printing.fasl")
(#x949404f0f5c1b8e8aa7fc852d0fe5e1a . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x88c0a80f4fb3dc1f9defef29b58b5a39 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xaa93883d6c3b6a246786e8304ebd10a0 . "./code.cg2js/cg-pixmap.fasl")
(#x99565667993ac9bae0f2d748b36fb88f . "./code.cg2js/cg-plot-widget.fasl")
(#x0d48f6bc1df2a7aa6bd3af6bab6990a1 . "./code.cg2js/cg-printing.fasl")
(#x3f0cabde0d24857bedba0a6c4e0232be . "./code.cg2js/cg-progress-indicator.fasl")
(#xda9dd23ff1712adf5f7e1077d89c3d9a . "./code.cg2js/cg-project-window.fasl")
(#x7f21233ccdf938e6cd722ef14d76e25d . "./code.cg2js/cg-property.fasl")
(#xd133725a9fd9e6394104ac405bce6882 . "./code.cg2js/cg-radio-button.fasl")
(#x2dd97b2ff82e408ad5797e87f1200e5c . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x2db7b78bdabb98754cf18cb465e3efaf . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xe8ab80ae53875b5afc715d13cebc6729 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xea4295ff7ad6823106c5d625a1736680 . "./code.cg2js/cg-rich-edit.fasl")
(#x97cd400048723a4f5a8cc928e25ad5f8 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x4e17ff03eb03b93c9e0ac1e4a73decaa . "./code.cg2js/cg-scaling-stream.fasl")
(#x2e364e1e433a6398adf0a95491848d5e . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x9dfe03ebcdf5c87d22825c613321ae51 . "./code.cg2js/cg-scroll-bar.fasl")
(#xe1891caccbae89940de18147201d28ba . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x3da03a1214098ab494aa53dc55a985a5 . "./code.cg2js/cg-selected-object.fasl")
(#x51e0ca7c7f5704bc49add126dd7bede8 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xca3cc87db43abf177fd5689de1823813 . "./code.cg2js/cg-split-bar.fasl")
(#xc8df2b1dbb9cf3bdefee03437b309258 . "./code.cg2js/cg-static-text.fasl")
(#x9438ed04443194ebf3f1534ce53808cf . "./code.cg2js/cg-status-bar.fasl")
(#x7062049ce4e3c1bc27b21cfa27173400 . "./code.cg2js/cg-string-dialog.fasl")
(#xe0d520bf60013d0f0241c8fd949af57f . "./code.cg2js/cg-tab-control.fasl")
(#xdc87caa0cdbca8176cc67fc6aeea4775 . "./code.cg2js/cg-template-string.fasl")
(#xfd252092107a68761261bc3bd65e13e1 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xf0f293dca35f2b3966229ea381648772 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x1023a1c814682ef082453f1ed27f07e5 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x49fc1e6c2341a019c471b70a1bf95674 . "./code.cg2js/cg-text-or-combo.fasl")
(#x1b572138fe98dcf3053761a264f4818e . "./code.cg2js/cg-text-widget.fasl")
(#x23023ccbc821296f70073deb1f99efbe . "./code.cg2js/cg-timer.fasl")
(#xcde6f3cef1cc6e7d7ab67b97088f633c . "./code.cg2js/cg-toggling-widget.fasl")
(#x93a843959c21ed6b2fc4cb6a3ec03f1d . "./code.cg2js/cg-toolbar.fasl")
(#x7e5e2e845cd0d0c62f0cdfecb112c9cf . "./code.cg2js/cg-tooltip.fasl")
(#xe903e314f20b868106a502ffcc464df8 . "./code.cg2js/cg-touch.fasl")
(#x378d4a5981da282433f7db7a5b6f6950 . "./code.cg2js/cg-trackbar.fasl")
(#xac23c829ed55a4f2459be1a19fa06ddf . "./code.cg2js/cg-tray.fasl")
(#x05dce5e920d1a17794e27fddfd25ee09 . "./code.cg2js/cg-up-down-control.fasl")
(#xcbc4d1e454ff4aafe7474e80f2b500f3 . "./code.cg2js/cg-utility-dialog.fasl")
(#x13f35cea4bdf30047d12cda1e84a4851 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x99459a216226bdf8e6657a53c759d1bf . "./code.cg2js/cg-web-browser.fasl")
(#xd590e69c87628c226ac54c0e9e997dfd . "./code.cg2js/cg-wrap-string.fasl")
(#xa2332d7ce873ec22f0c1797ef4564ab7 . "./code.cg2js/cg-yes-no-list.fasl")
(#x27567c04115b40fd383a7b52d0ba1380 . "./code.cg2js/cg-yes-no-string.fasl")
(#x0defe1be72b7fcf999a06d037db35299 . "./code.cg2js/cg.fasl")
(#x6488ead4d4afcb5ecf7adee57fe59250 . "./code.idejs/ide-acache.fasl")
(#x5cd33b5eabde00a9575a3ba1b7f359f2 . "./code.idejs/ide-apropos.fasl")
(#x26eca3169cf11ef6f6321e46e881c0f0 . "./code.idejs/ide-base.fasl")
(#xca44663f1a8c2cafbfcbca793f45c375 . "./code.idejs/ide-class-browser.fasl")
(#x21d4999252f6d8f88606c08b15a8b7f7 . "./code.idejs/ide-class-grapher.fasl")
(#x7623ab800cbde43b356b695c0e1eb299 . "./code.idejs/ide-clipboard.fasl")
(#x9f2e05a0f9fe9c98657275f9eb1a998a . "./code.idejs/ide-dev.fasl")
(#xbdaf64c27bee52958110fd234d4b5bc3 . "./code.idejs/ide-editor.fasl")
(#x0d8f8fda71cd96eca2d9d0e3c758d705 . "./code.idejs/ide-find-definitions.fasl")
(#xd54faa2443b31aa83170a13fccaefa39 . "./code.idejs/ide-find-in-files.fasl")
(#xbf23521a5cc6db425593e3fb3a7bd8ab . "./code.idejs/ide-form.fasl")
(#x50738a4e2185716a2008842cd89a44e3 . "./code.idejs/ide-grapher.fasl")
(#x0ead67e14fe54a9eca8bebd7db72cc34 . "./code.idejs/ide-inspector.fasl")
(#x0d2b8bea038322ca2a6058070753085c . "./code.idejs/ide-intro.fasl")
(#x43a4d8d60fa40422204a9bfa696a7626 . "./code.idejs/ide-keyshort.fasl")
(#xa8f839f4b6c63f773109f6e80883b9d7 . "./code.idejs/ide-navigator.fasl")
(#xb1c6cafd4385241f27519923130e71da . "./code.idejs/ide-package-list.fasl")
(#xa13c94f3389bacbaa34d3668272ac547 . "./code.idejs/ide-patch-dialog.fasl")
(#xdcbe7f4e14c2730e429469049df43aca . "./code.idejs/ide-process-browser.fasl")
(#x63c0fae55e71fc3b1ea769136f3b4940 . "./code.idejs/ide-project.fasl")
(#xa2f3ededf6341cb7980dc42a2e096254 . "./code.idejs/ide-trace.fasl")
(#x83445e4875656760032a1c4c0f2d203d . "./code.idejs/ide-window-list.fasl")
(#x6fc53b932fdc4c82ffe3c3c1eae9d4b5 . "./code.idejs/ide.fasl")
(#xdf73f3983b983ba8030e7ee4702b498a . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
